#!/bin/sh

# those should be set by user
irc_prefix="/home/labla/irc"
less_keys="/home/labla/iil/iilkeys"
less_cmd="less -k$less_keys -M +G"
read_cmd="read -p ; "

# global variables, rest should be local
# no real usage, beside keeping things tight
server_list=""
channel_list=""
last_server=0
last_channel=0
current_path=""
current_tag=0
exit_hook=""

# trap INT, so +F (forw-forever) in less doesn't break $exit_hook

trap ":" INT

get_list()
{
  # get list with full server pathes
  # e.g. $server_list="/home/user_name/irc/irc.oftc.net ..."

  server_list=`du -d 1 $irc_prefix | awk -v irc_prefix=$irc_prefix '
    { if ($2 != irc_prefix) print $2 }'`

  # find last entry in server list and negate it

  set -- $server_list
  last_server=$((0 - $#))

  # get list with full channels pathes
  # e.g. $channel_list="/home/user_name/irc/irc.oftc.net/\#suckless ..."
  # we need to get also private messages so it is more
  # compilecated than /#/ { print }

  channel_list=`du $irc_prefix | awk -v irc_prefix=$irc_prefix '
    BEGIN {
      FS="/";
      max = split(irc_prefix, irc_split, "/");
    }

    NF > max + 1 {
      for (i = 2; i <= NF; i++) printf("/%s", $i);
      print "";
    }'`
  
  # find last entry in channel list

  set -- $channel_list
  last_channel=$#

  # lint
  # echo "server list:" $last_server $server_list
  # echo "channel list:" $last_channel $channel_list
}

show_tag()
{
  # check if $current_tag is proper integer, otherwise set to 0
  
  if [ $current_tag -eq $current_tag 2> /dev/null ] ; then
    current_tag=$current_tag
  else
    current_tag=0
  fi

  # case where $current_tag could be empty e.g. obtained via send_msg
  
  if [ -z $current_tag ] ; then
    current_tag=0
  fi

  # check if $current_tag is in range of $last_server..$last_channel
  # otherwise wrap to first ($last_server) or last item ($last_channel)
  
  if [ $current_tag -gt $last_channel ] ; then
    current_tag=$last_channel
  elif [ $current_tag -lt $last_server ] ; then
    current_tag=$last_server
  fi

  # lint
  # echo "current tag after parsing:" $current_tag

  # depending on value of $current_tag use less to show
  # >0 specific channel
  # <0 specific server
  # =0 list of servers and channels

  if [ $current_tag -gt 0 ] ; then
    set -- $channel_list
    eval current_path=\$$current_tag
    $less_cmd ${current_path}/out
    exit_hook=$?
  elif [ $current_tag -lt 0 ] ; then
    set -- $server_list
    eval current_path=\$$(($current_tag * -1))
    $less_cmd ${current_path}/out
    exit_hook=$?
  else
    current_path=""
    show_list
  fi

  # lint
  # echo "current path:" $current_path
}

show_list()
{
  (
    # print server list in reverse order
  
    local j=$(($last_server * -1))
    set -- $server_list

    while [ $j -gt 0 ] ; do
      eval echo "["$((0 - $j))".]" \$$j
      j=$(($j - 1))
    done

    printf "\n[ 0.] Show/refresh list and optionally switch to specific tag\n\n"

    # print channel list

    local i=1
    set -- $channel_list

     while [ $i -le $last_channel ] ; do
      eval echo "[ "${i}".]" \$$i 
      i=$(($i + 1))
    done

  ) | $less_cmd

  exit_hook=$?
}

send_msg()
{
  # depending on $current_tag
  # =0 change channel/server
  # !=0 send message to channel/server

  local msg

  if [ $current_tag -ne 0 ] ; then
    printf "Send msg to: %s/in\n" "${current_path}"
    $read_cmd msg
    if [ -n "$msg" ] ; then
      printf "%s\n" "$msg" > ${current_path}"/in"
    fi
    show_tag $current_tag
  else
    printf "Change channel/server to:\n"
    $read_cmd msg
    current_tag=$msg
    show_tag $current_tag
  fi
  
  # lint            
  # echo "msg:" $msg 
}

loop()
{
  # lint
  # echo "exit_hook:" $exit_hook

  # value of $exit_hook is a ascii character
  # check awk 'BEGIN { printf "%c\n", 44 }'
  # those can be found in lesskeys:
  # 44 = ,
  # 46 = .
  # 59 = ;
  # 48..57 = 0..9
  
  while : ; do

    case $exit_hook in
      44)
        current_tag=$(($current_tag - 1))
        show_tag $current_tag  
        ;;
      46)
        current_tag=$(($current_tag + 1))
        show_tag $current_tag
        ;;
      59)
        send_msg
        ;;
      48)
        get_list
        current_tag=0
        show_tag $current_tag
        ;;
      49)
        current_tag=1
        show_tag $current_tag
        ;;
      50)
        current_tag=2
        show_tag $current_tag
        ;;
      51)
        current_tag=3
        show_tag $current_tag
        ;;
      52)
        current_tag=4
        show_tag $current_tag
        ;;
      53)
        current_tag=5
        show_tag $current_tag
        ;;
      54)
        current_tag=6
        show_tag $current_tag
        ;;
      55)
        current_tag=7
        show_tag $current_tag
        ;;
      56)
        current_tag=8
        show_tag $current_tag
        ;;
      57)
        current_tag=9
        show_tag $current_tag
        ;;
      *) break
        ;; 
    esac

  done  

}

# body of script begins here

get_list

# check for argument passed to script and set $current_tag

if [ $1 ] ; then
  current_tag=$1
else
  current_tag=0
fi

# lint
# echo "current tag from argument:" $current_tag

show_tag $current_tag
loop

